var cumin;

(function() {
    cumin = new Cumin();

    function runModelListeners(model) {
        for (var id in this.modelListeners) {
            this.modelListeners[id](id, model);
        }
    }

    function runObjectListeners(object) {
        for (var id in this.objectListeners) {
            this.objectListeners[id](id, object);
        }
    }

    function Cumin() {
        this.modelListeners = new Object();
        this.objectListeners = new Object();

        this.runModelListeners = runModelListeners;
        this.runObjectListeners = runObjectListeners;

        this.refreshTime = function(src) {
            var branch = wooly.session.branch(src);
            branch.ts = new Date().getTime();
            return branch.marshal()
        }

        this.updateChart = function(id) {
            var chart = $(id);
            var img = chart.getElementsByTagName("img")[0]

            var src = img.src;
            src = cumin.refreshTime(src);
            img.src = src;
        }

        this.clickTableCheckbox = function (check, name) {
            var val = check.value;
            var hash = wooly.session.hash();
            if (!(name in hash)) {
                hash[name] = "";
            }
            var vals = Array.from(hash[name].split('|'));
            if ((vals.length == 1) && (vals[0] == ""))
                vals.empty();
            var valIndex = vals.indexOf(val);
            var negValIndex = vals.indexOf("-"+val);
            if (check.checked) {
                // the checkbox was just checked
                if (negValIndex > -1)
                    vals.erase("-"+val);
                else if (valIndex == -1)
                    vals.push(val);
            } else {
                if (valIndex > -1)
                    vals.erase(val);
                else if (negValIndex == -1)
                    vals.push("-"+val);
            }
            if (vals.length == 0)
                delete hash[name];
            else
                hash[name] = vals.join('|');
            wooly.session.setHash(hash);
        }

        this.restoreTableCheckboxes = function () {
            var hash = wooly.session.hash();
            for (var key in hash) {
                if (!(hash[key] instanceof Function)) {
                    if (document.forms.length > 0) {
                        var checks = document.forms[0].elements[key];
                        if (checks) {
                            var vals = Array.from(hash[key].split('|'));
                            if ((vals.length == 1) && (vals[0] == ""))
                                vals.empty();
                            if (typeof checks.length == "undefined") {
                                if (vals.contains(checks.value)) {
                                    checks.checked = true;
                                }
                                if (vals.contains("-"+checks.value))
                                    checks.checked = false;
                            } else {
                                for (var i=0; i < checks.length; i++) {
                                    if (vals.contains(checks[i].value)) {
                                        checks[i].checked = true;
                                    }
                                    if (vals.contains("-"+checks[i].value))
                                        checks[i].checked = false;
                                }
                            }
                        }
                    }
                }
            }
        }

        this.setFullpageHandler = function (id, fullpage_url) {
            var chart = $(id);
            if (chart) {
                chart.preFullPage = function () { cumin.fullpageChart(fullpage_url); return false;};
            }
        }

        this.setupChart = function (id, width) {
            var chart = $(id);
            chart.onfullpage = function (width, height) { cumin.chartNotify(true, width, height, id); };
            chart.onrestore = function () { cumin.chartNotify(false, width, 100, id); };
            var oImg = document.images[id];
            var mImg = $(oImg);
            mImg.addEvent('load', function () {
                this.style.visibility = "visible";
                this.removeAttribute("width");
                this.removeAttribute("height");
                this.className = "";
                var loading = chart.getElement(".loading");
                loading.setStyle('display', 'none');
                loading.setStyle('visibility', 'visible');
                loading.loading = false;
                var over = mImg.retrieve("over");
                if (over) {
                    get_chart_points(oImg);
                }
            });
            mImg.addEvent('mousedown', function(event){
                event.stop();
            });
            mImg.addEvent('mouseout', function(event){
                if (event.relatedTarget && event.relatedTarget.className)
                    if (event.relatedTarget.className.indexOf("img") == 0) {
                        event.stop();
                        return false;
                    }
                var mBody = $(document.body);
                var oHighlight = mBody.getElement(".imgHighlight_red");
                if (oHighlight) oHighlight.style.display = "none";
                oHighlight = mBody.getElement(".imgHighlight_green");
                if (oHighlight) oHighlight.style.display = "none";
                oHighlight = mBody.getElement(".imgHighlight_blue");
                if (oHighlight) oHighlight.style.display = "none";
                var oValue = mBody.getElement(".imgValues_red");
                if (oValue) oValue.style.display = "none";
                oValue = mBody.getElement(".imgValues_green");
                if (oValue) oValue.style.display = "none";
                oValue = mBody.getElement(".imgValues_blue");
                if (oValue) oValue.style.display = "none";
                mImg.store("over", false);
            });
            mImg.addEvent('mouseover', function(event){
                mImg.store("over", true);
                get_chart_points(mImg);
            });
            mImg.addEvent('mousemove', function(event){
                cumin.chartMove(mImg, event);
            });
        }

        this.chartMove = function (mImg, event) {
            closestPoint = function (x, info) {
                samples = info['points'];
                var closest = samples[0];
                for (var i=0; i<samples.length; i++) {
                    var coord = samples[i];
                    if (Math.abs(x - coord[0]) < Math.abs(x -closest[0]))
                        closest = coord;
                }
                return closest;
            }
            highlight = function (stat, point, xy) {
                var mBody = $(document.body);
                var oHighlight = mBody.getElement(".imgHighlight_"+stat);
                var oValues = mBody.getElement(".imgValues_"+stat);
                if (!oHighlight) {
                    oHighlight = new Element('div', { 'class': 'imgHighlight_'+stat });
                    oValues = new Element('div', { 'class': 'imgValues_'+stat });
                    mBody.appendChild(oHighlight);
                    mBody.appendChild(oValues);
                }
                var pos = mImg.getPosition();
                oHighlight.style.display = "block";
                oHighlight.style.left = (pos.x + point[0] - 5) + "px";
                oHighlight.style.top = (pos.y + point[1] - 3) + "px";

                oValues.style.display = "block";
                oValues.style.left = (pos.x + xy[0] + 32) + "px";
                oValues.style.top = (pos.y + xy[1] - 8) + "px";
                oValues.innerHTML = "("+point[2]+")";
            }
            
            var points = mImg.retrieve('points');
            if (points) {
                for (var stat in points) {
                    var samples = points[stat];
                    var x = event.client.x;
                    mImg.store("lastX", x);
                    var pos = mImg.getPosition();
                    var point = closestPoint(x - pos.x, samples);
                    if (typeof point != "undefined")
                        highlight(points[stat]['color'], point, points[stat]['xy']);
                }
            }
        }

        // called when a chart widget is maximized or restored
        this.chartNotify = function (full, width, height, id) {
            var bWidth = Math.max(width - 100, 360);
            var bHeight = Math.max(height - 100, 100);
            var oImg = document.images[id];
            if (oImg) {
                var src = oImg.src;
                var branch = wooly.session.branch(src);
                branch.width = bWidth;
                branch.height = bHeight;
                src = branch.marshal();

                src = cumin.refreshTime(src);
                oImg.className = "Loading";
                oImg.onerror = oImg.onload = function () {this.className = ""};
                oImg.width = width;
                oImg.height = height;
                oImg.style.visibility = "hidden";
                oImg.src = src;

                var loading = $(id).getElement(".loading");
                loading.loading = true;
                setTimeout("showLoading('"+id+"')", 1000);
                setTimeout("hideLoading('"+id+"')", 1000 * 60);
            }
            return true;
        }

        this.makeFullPageable = function (element) {
            if (element.getParent().getElement(".fullpageIcon"))
                return; // already fullpaged

            var fpIconClass = 'fullpageIcon';
            if (element.get('class').indexOf("oldfpiconlocation") != -1) {
            	fpIconClass = 'fullpageIconOldLocation';
            }
            
            var icon = new Element('p', {'class': fpIconClass, 
                 'title': 'Full Page', 
                 'events': {'click': function () {fullpage(this)}}});

            var title = element.getElement("h2");
            if (title) {
                icon.inject(title, 'before');
                title.addEvent('dblclick', function () {fullpage(icon)});
            }
            element.originalParent = element.getParent();

            var widgets = element.getElements('.fullpage_notify');
            element.widgetPaths = [];
            widgets.each(function (widget) {
                if (widget.get("id")) {
                    element.widgetPaths.push(widget.get("id"));
                }
            });
        }

        this.fullpageChart = function (fullpage_url) {
            var params  = 'width='+screen.width;
            params += ', height='+screen.height;
            params += ', top=0, left=0'
            params += ', fullscreen=yes';
            var branch = wooly.session.branch(fullpage_url);
            branch['width'] = Math.floor(screen.width * 0.96);
            branch['height'] = Math.floor(screen.height * 0.96);

            var newwin = window.open(branch.marshal(), 'cuminflash', params);
            if (window.focus) {
                newwin.focus();
            }
        }

    }
}())


/* surround all elements that have class 'fullpageable' with
    supporting divs and add event behaviors */
window.addEvent('domready', function () {
    var elements = $$('.fullpageable');
    for (var i=0; i<elements.length; i++) {
        var element = elements[i];

        if (element.getStyle("visibility") != "hidden") {
            cumin.makeFullPageable(element);
        }
    }
});

/* called when the full page icon is clicked */
function fullpage(oIcon) {
    var oFullPage = oIcon.getParent();

    // short circuit to open in a new window
    if (oFullPage.preFullPage) {
        if (!oFullPage.preFullPage())
            return false;
    }

    var oBack = $("fullpageBack");
    if (!oBack) {
        oBack = new Element('div', {'class': 'fullpageBack', 'id': 'fullpageBack', 'events': {'click': fullpage}});
        document.body.appendChild(oBack);
    }
    if (oFullPage.parentNode == document.body) {
        // undo full page
        oBack.style.display = "none";
        oFullPage.removeClass('fullpaged');
        oFullPage.addClass('fullpageable');

        document.body.style.height = "auto";
        oFullPage.originalParent.appendChild(oFullPage);
        oIcon.setProperty("title", "Full Page");
        setFullpageParam(false, oFullPage);
        var onrestore = oFullPage.onrestore;
        if (onrestore) onrestore();
        window.scrollTo(oFullPage.origScroll.x, oFullPage.origScroll.y);
    } else {
        // make it full page

        oBack.style.display = "block";
        oBack.style.height = Math.max($(document.body).getCoordinates().height, window.getScrollSize().y) + "px";
        document.body.style.height = "100%";

        document.body.appendChild(oFullPage);

        oFullPage.removeClass('fullpageable');
        oFullPage.addClass('fullpaged');

        oIcon.setProperty("title", "Restore");
        setFullpageParam(true, oFullPage);

        var onfullpage = oFullPage.onfullpage;
        var coords = $(oFullPage).getCoordinates();
        var height = Math.min(Math.max(window.getSize().y - 100, 200), 600);
        if (onfullpage) 
            onfullpage(coords.width, height);

        var curScroll = window.getScroll();
        oFullPage.origScroll = curScroll;
        window.scrollTo(0,0);
    }

    // sets/clears the fullpage parameter in the wooly background update url 
    function setFullpageParam(set, oFullPage) {
        var branch = wooly.branchIntervalUpdate();
        if (set) {
            var width = $(oFullPage).getCoordinates().width + "";
            oFullPage.widgetPaths.each(function (path) {
                branch.session[path+".fullpage"] = width;
            });
        } else {
            oFullPage.widgetPaths.each(function (path) {
                delete branch.session[path+".fullpage"];
            });
        }
        wooly.restartIntervalUpdate(branch.marshal());
        cumin.expireIntervalUpdate();
    }
}

cumin.getFlashChart = function (id) {
    var chart = $(id+"_chart");
    return chart;
}

cumin.updateFlashChart = function (id) {
    var chart = cumin.getFlashChart(id);
    if (chart == null)
        return false;
    if (typeof chart.get_ymax != "undefined")
        chart.get_ymax(id)
    else
        wooly.log("get_ymax not definded for " + id);
}

/* called from .swf in response to chart.get_ymax() */
function got_ymax(id, href, axis_max, vals_max, low, high) {
    var chart = cumin.getFlashChart(id);
    if (chart == null)
        return false;

    var branch = wooly.session.branch(href);

    var now = new Date().getTime();
    var then = now - 10 * 1000;
    if (typeof chart.last_time != "undefined") {
        then = chart.last_time;
    }
    chart.last_time = now;
    var elapsed = now - then;
    branch['elapsed'] = elapsed; // milliseconds since last update

    // hack: do a full update every minute
    if (typeof chart.last_full_time == "undefined") {
        chart.last_full_time = now;
    }
    var last_full_time = chart.last_full_time;
    if (now - last_full_time > 60 * 1000) {
        if (low != "-1") {
            branch['low'] = low;
            branch['high'] = high;
            delete branch.elapsed;
        }
        chart.last_full_time = now;
    }

    branch['amax'] = axis_max;
    branch['vmax'] = vals_max;

    href = branch.marshal();
    if (typeof chart.reload != "undefined")
        chart.reload(href, false);
    else
        wooly.log("reload not definded for " + id);
}

function incrementChartSequence(chart) {
    var src = chart.src;
    var branch = wooly.session.branch(src);
    if (typeof branch['seq'] == "undefined")
        branch['seq'] = '0';
    var seq = parseInt(branch['seq']) + 1;
    branch['seq'] = seq;
    chart.src = branch.marshal();
    return seq;
}


/*
function ofc_ready(id) {
    if (typeof id != "undefined") {
        var key = id;
        var hash = wooly.session.hash();
        if (typeof hash[key] != "undefined") {
            var oChartHtml = $(key);
            if (oChartHtml) {
                var links = oChartHtml.getElements('a');
                var state = hash[key];

                $each(links, function (item) {
                    if (item.get('state') == state) {
                        changeDuration(state, item, key);
                    }
                });
            }
        }
    }
}

window.addEvent('domready',function () {
    var hash = wooly.session.hash();
    for (var key in hash) {
        if (!(hash[key] instanceof Function)) {
            var oImg = document.images[key];
            if (oImg) {
                var oChart = $(key);
                if (oChart) {
                    var links = oChart.getElements('a');
                    var state = hash[key];

                    $each(links, function (item) {
                        if (item.get('state') == state) {
                            changeDuration(state, item, key);
                        }
                    });
                }
            }
        }
    }
});
*/

function ofc_range(id, href, low, high) {
    var chart = cumin.getFlashChart(id);
    if (chart) {
        var now = new Date().getTime();
        chart.last_full_time = now;

        var branch = wooly.session.branch(href);

        // don't allow high < low
        var iLow = parseFloat(low, 10);
        var iHigh = parseFloat(high, 10);
        if (iHigh < iLow) {
            low = high;
            high = iLow + "";
        }

        branch['low'] = low;
        branch['high'] = high;
        if (branch.elapsed)
            delete branch.elapsed;

        href = branch.marshal();
        if (typeof chart.reload != "undefined")
            chart.reload(href, true);
        else
            wooly.log("reload not definded for " + id);
    }
}

function ofc_change_priority(index, text) {
    var fields = text.split("|");
    //var percent = fields[0].substring(0, fields[0].length - 1);
    //var value = fields[1];
    var label = fields[2];
    //var id = fields[3];

    var selinput = null;
    $$(".slider").each( function (el, i) {
        var input = el.getParent("tr").getElement("input");
        if (el.id == label) {
            selinput = input;
        } else {
            input.selectRange(0, 0);
        }
    });
    if (selinput) {
        selinput.focus();
        selinput.selectRange(0, selinput.value.length);
    }
}

function addJavascript(jsname, pos) {
    var th = document.getElementsByTagName(pos)[0];
    var s = document.createElement('script');
    s.setAttribute('type','text/javascript');
    s.setAttribute('src',jsname);
    th.appendChild(s);
}
addJavascript('resource?name=incrementalSearch.js', 'head');

function ofc_debug(msg) {
    wooly.log(msg);
}

function showHistoryChart(title, src) {

    if (src != '') {
        new StickyWin.Modal.Ajax({
          url: src,
          wrapWithUi: true,
          caption: title,
          useIframeShim: false,
          allowMultiple: false,
          closeOnEsc: true,
          uiOptions: {width: 420},
          edge: false,
          position: 'center',
          offset: {x:0,y:0},
          relativeTo: document.body
        }).update();
    }
    return false;
}
